
struct CSMFetch
{

	matrix shadowSpaceProjection;
	float splitPoint;
};

matrix View : VIEWMATRIX;
matrix projInverse : SCREEN_TO_WORLD;

float4 vLightDir[2] : LIGHTDIRECTION;
float4 vLightColor[2] : LIGHTCOLOR;
int lightCount : COUNT_LIGHTDIRECTION;

CSMFetch split[8] : CSMLookupInformation;
int splitCount : COUNT_CSMLookupInformation;
Texture2D shadowSources[8] : RT_CONST_CSMMap;

Texture2D posTex : RT_CONST_POSITIONTEX;
Texture2D normTex : RT_CONST_NORMTEX;
Texture2D colorTex : RT_CONST_COLORTEX;
Texture2D ssaoTex : RT_CONST_SSAO;
Texture2D simpleRefract : RT_CONST_REFRACT;

SamplerState simpleSampler : SAMPLER
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

SamplerState directSampler : SAMPLER
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct PS_VARY
{

    float4 vPos : SV_POSITION;
    float2 tCoord : TEXCOORD;
    float2 screenPos : SPOS;
};

PS_VARY VS ( float4 Pos : POSITION, float2 tC : TEXCOORD )
{

    PS_VARY output;
    
    output.vPos = Pos;
    output.tCoord = tC;
    output.screenPos = Pos.xy;
    
    return output;
}

void PS( PS_VARY vSrc, out float4 firstTarget : SV_Target0 )
{

	float4 worldPos;
	worldPos.z = posTex.Sample( simpleSampler, vSrc.tCoord ).x;
	worldPos.xy = vSrc.screenPos;
	worldPos.w = 1.0f;
	worldPos = mul( worldPos, projInverse );
	worldPos /= worldPos.w;
	
    float3 refractNormal = simpleRefract.Sample( simpleSampler, vSrc.tCoord ).xyz * 2.0f - float3( 1.0f, 1.0f, 1.0f );
    float refractOffset = 1.0f / refractNormal.z - 1.0f;
    vSrc.tCoord += refractOffset * refractNormal.xy / 30.0f;
    
    float3 normal = normTex.Sample( simpleSampler, vSrc.tCoord ) * 2.0f - float3( 1.0f, 1.0f, 1.0f );
    float4 txColor = colorTex.Sample(simpleSampler, vSrc.tCoord );
    
    firstTarget = float4( 0, 0, 0, 0 );
    for(int i=0; i<lightCount; i++)
    {
    
		firstTarget += saturate( dot( (float3)vLightDir[i], normal ) * vLightColor[i] ) * txColor;
    }
    
    bool search = true;
    float multiplier = 1.0;
    for( int index = 0; index < splitCount && search; index++ )
    {
    
		if ( mul( worldPos, View ).z <= split[ index ].splitPoint.x )
		{
			
			matrix lightSpaceTransform = transpose( split[ index ].shadowSpaceProjection );
 			
 			float3 lightPos = mul( worldPos, lightSpaceTransform );
		
			if ( shadowSources[ index ].Sample( simpleSampler, lightPos.xy ).x + 0.001f < lightPos.z ) multiplier = 0.0f;
			search = false;
		} 
    }
    
    firstTarget *= multiplier;
    firstTarget += float4( 0.5f, 0.5f, 0.3f, 1.0f ) * txColor * pow( ssaoTex.Sample( simpleSampler, vSrc.tCoord ).x, 5.0f );
    firstTarget *= refractNormal.z;
    //firstTarget = pow( ssaoTex.Sample( simpleSampler, vSrc.tCoord ).x, 5.0f );
}

technique10 Render
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, PS() ) );
    }
}
