
matrix normalMat : NORMALMATRIX;
matrix world : WORLDMATRIX;
matrix projection : WORLDVIEWPROJMATRIX;
Texture2D ambientTex : COLORTEXTURE;

SamplerState simpleSampler : SAMPLER
{
    Filter = MIN_MAG_MIP_LINEAR;
    AddressU = Wrap;
    AddressV = Wrap;
};

struct VARYING
{

	float4 vPos : SV_POSITION;
	float3 wPos : WORLDPOS;
    float3 Norm : NORMAL;
    float2 tCoord : TEXCOORD;
};

void Deferred_VS ( float4 Pos : POSITION, float3 Norm : NORMAL, float2 texCoord : TEXCOORD, out VARYING vs_output )
{

    vs_output.vPos = mul( Pos, projection );
    vs_output.wPos = mul( Pos.xyz, (float3x3) world );
    vs_output.Norm = mul( -Norm, (float3x3) normalMat );
    vs_output.tCoord = texCoord;
    vs_output.tCoord.y = 1 - texCoord.y;
}

void Deferred_PS ( in VARYING ps_input, out float3 normals : SV_Target0, out float3 colors : SV_Target1 )
{

	normals = normalize( ps_input.Norm ) / 2.0f + float3( 0.5f, 0.5f, 0.5f );
	colors = ambientTex.Sample( simpleSampler, ps_input.tCoord );
}

technique10 DefaultDeferredShading
{
    pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, Deferred_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, Deferred_PS() ) );
    }
}