
// SSAO PASS
Texture2D depthMap : RT_CONST_SSAODEPTH;
Texture2D random : RANDOMSOURCE;
int sampleCount : SAMPLES;
float sampleRadius : RADIUS;
float threshold : MAX_DEPTH_DIFF;
matrix projectionMatrix : PROJECTIONMATRIX;
matrix expToLin : EXP_TO_LIN;

SamplerState directSampler : SAMPLER_DIRECT
{

	Filter = MIN_MAG_MIP_POINT;
	AddressU = Wrap;
    AddressV = Wrap;
};

struct QuadStructEye
{

	float4 vPos : SV_POSITION;
	float2 tCoord : NORMAL;
	float3 viewDir : VIEW;
};

QuadStructEye SSAO_RandomSample_VS ( float4 Pos : POSITION, float3 eyeVec : VIEWVECTOR, float2 texCoord : TEXCOORD )
{

	QuadStructEye output;
	output.vPos = Pos;
	output.tCoord = texCoord;
	output.viewDir = eyeVec;
	
	return output;
}

void SSAO_RandomSample_PS ( in QuadStructEye quadSrc, out float ambientOcclusion : SV_Target0 )
{

	float2 expDepth = float2( depthMap.Sample( directSampler, quadSrc.tCoord ).x, 1 );
	expDepth = mul( expDepth, expToLin );
	float currentSampleDepth = - expDepth.y / expDepth.x;
 	float3 viewSpacePosition = currentSampleDepth * quadSrc.viewDir;
 	
 	float3 planeVector = random.Sample( directSampler, quadSrc.tCoord * 10 );
 	planeVector = normalize( planeVector );
 	
 	int validSampleCount = 0;
 	ambientOcclusion = 0;
 
 	for ( int sample_index = 0; sample_index < sampleCount; sample_index++ )
 	{
 	
 		float3 randomSample;
 		randomSample.x = sin( sample_index ) * cos( sample_index * 8 );
 		randomSample.y = sin( sample_index ) * sin( sample_index * 8 );
 		randomSample.z = cos( sample_index );
 		
 		randomSample = reflect( randomSample, planeVector );
 		float vLength = abs( sin( sample_index * 2 ) );
 		randomSample *= sampleRadius * vLength;
 		 		
 		float4 randomSamplePosition = float4( viewSpacePosition + randomSample, 1 );
 		randomSamplePosition = mul( randomSamplePosition, projectionMatrix );
 		randomSamplePosition /= randomSamplePosition.w;
 		randomSamplePosition.x = 0.5f * randomSamplePosition.x + 0.5f;
 		randomSamplePosition.y = -0.5f * randomSamplePosition.y + 0.5f;
 		
 		float2 compareSample = float2( depthMap.Sample( directSampler, randomSamplePosition.xy ).x, 1 );
 		float2 randomFetchSample = float2( randomSamplePosition.z, 1 );
 		
 		compareSample = mul( compareSample, expToLin );
 		randomFetchSample = mul( randomFetchSample, expToLin );
 		compareSample /= -compareSample.x;
 		randomFetchSample /= -randomFetchSample.x;
 		
 		float diff = randomFetchSample.y - compareSample.y; // if diff is positive, the neighbour pixel is closer, thus the pixel is occluded
 		
 		if ( diff < threshold )
 		{
 		
 			if ( diff > 0 )	ambientOcclusion += 1.0f - vLength;//pow( ( 1.0f - vLength ), 2 );
  			validSampleCount++;
 		}
 	}
 	
 	ambientOcclusion /= validSampleCount;
 	ambientOcclusion = 1 - ambientOcclusion;
	if ( validSampleCount == 0 ) ambientOcclusion = 0.5f;
}

technique10 SSAO_RandomApproach_Perlin
{

	pass P0
    {
        SetVertexShader( CompileShader( vs_4_0, SSAO_RandomSample_VS() ) );
        SetGeometryShader( NULL );
        SetPixelShader( CompileShader( ps_4_0, SSAO_RandomSample_PS() ) );
    }
}